/*******************************************************************************
  Main Source File

  Company:
    Microchip Technology Inc.

  File Name:
    main.c

  Summary:
    This file contains the "main" function for a project.

  Description:
    This file contains the "main" function for a project.  The
    "main" function calls the "SYS_Initialize" function to initialize the state
    machines of all modules in the system
 *******************************************************************************/

// *****************************************************************************
// *****************************************************************************
// Section: Included Files
// *****************************************************************************
// *****************************************************************************

#include <stddef.h>                     // Defines NULL
#include <stdbool.h>                    // Defines true
#include <stdlib.h>                     // Defines EXIT_FAILURE
#include "definitions.h"                // SYS function prototypes
#include <stdio.h>
#include <string.h>
// TODO 17.01
#include "../Lab17_SERCOM_SPI_OLED_SSD1306.X/u8g.h"

// *****************************************************************************
// *****************************************************************************
// Section: Main Entry Point
// *****************************************************************************
// *****************************************************************************
uint32_t i = 0;

volatile uint8_t TC3_IsOverflow = 0;
volatile uint8_t TC4_IsOverflow = 0;
uint8_t USARTRTxBuffer[100];
uint8_t USART5_ReceiveData[1];
volatile uint8_t USART5_IsReceived = 0;
volatile uint8_t USART5_IsTransmitted = 1;
uint16_t ADC_Result[2];
volatile uint8_t ADC_IsCompleted = 0;
volatile int8_t ADC_ChannelIdx = 0;
uint8_t Duty = 50;
int8_t DutyDistance = 2;

// TODO 17.02
u8g_t OLED_Instance;
extern const uint8_t Elite_Logo[];
uint8_t DisplayBuffer[4][64];

void ADC_Complete( ADC_STATUS status, uintptr_t context )
{		
	if( status & ADC_INTFLAG_RESRDY_Msk )
	{		
		ADC_Result[ADC_ChannelIdx] = ADC_ConversionResultGet();	
		ADC_ChannelIdx = ( ADC_ChannelIdx + 1 ) % 2;
		if( ADC_ChannelIdx==0 )
		{
			ADC_IsCompleted=1;
		}
	}
}

void USART5_Transmit( uintptr_t context )
{
	USART5_IsTransmitted = 1;
}

void USART5_Receive( uintptr_t context )
{
	USART5_IsReceived = 1;
}

void TC3_Overflow(TC_COMPARE_STATUS status, uintptr_t context) /* CallBack */
{
	if( status & TC_INTFLAG_OVF_Msk )
	{	
		TC3_IsOverflow = 1;
	}
}

void TC4_Overflow(TC_COMPARE_STATUS status, uintptr_t context) /* CallBack */
{
	if( status & TC_INTFLAG_OVF_Msk )
	{
		TC4_IsOverflow = 1;
	}
}

int main ( void )
{
	/* Initialize all modules */
	SYS_Initialize ( NULL );

	TC3_CompareCallbackRegister( TC3_Overflow, (uintptr_t )NULL );
	TC3_CompareStart();

	TC4_CompareCallbackRegister( TC4_Overflow, (uintptr_t )NULL );
	TC4_CompareStart();	

	SERCOM5_USART_ReadCallbackRegister( USART5_Receive, (uintptr_t )NULL );
	SERCOM5_USART_WriteCallbackRegister( USART5_Transmit, (uintptr_t )NULL );
	SERCOM5_USART_Read( USART5_ReceiveData, 1 );

	ADC_CallbackRegister( ADC_Complete, (uintptr_t )NULL );
	ADC_Enable();

	TCC2_PWMStart();
	
	// TODO 17.03
	SYSTICK_TimerStart();

	u8g_InitHWSPI( &OLED_Instance, &u8g_dev_ssd1306_128x64_hw_spi );
	u8g_SetFont( &OLED_Instance, u8g_font_6x10 );
	u8g_SetFontRefHeightExtendedText( &OLED_Instance );
	u8g_SetDefaultForegroundColor( &OLED_Instance );
	u8g_SetFontPosTop( &OLED_Instance );

	// TODO 17.04
	u8g_FirstPage( &OLED_Instance );
	do
	{
		u8g_DrawBitmap( &OLED_Instance, 0, 0, 16, 64, Elite_Logo );
		u8g_DrawLine( &OLED_Instance, 5, 60, 118, 60 );
		u8g_DrawBox( &OLED_Instance, 120, 59, 3, 3 );
		u8g_DrawBox( &OLED_Instance, 124, 59, 3, 3 );
	}
	while ( u8g_NextPage( &OLED_Instance ) );

	SYSTICK_DelayMs( 2000 );

	while ( true )
	{
		/* Maintain state machines of all polled MPLAB Harmony modules. */
		SYS_Tasks ( );

		if (TC3_IsOverflow)
		{
			TC3_IsOverflow = 0;
			LED1_Toggle();
			printf( "Hello World!!\r\n" );

			ADC_ConversionStart();

			// TODO 17.05
			sprintf( (char *) DisplayBuffer[0], "Hello World!!" );
			u8g_FirstPage(&OLED_Instance);
			do
			{
				u8g_DrawStr( &OLED_Instance, 0,  0, DisplayBuffer[0] );
				u8g_DrawStr( &OLED_Instance, 0, 10, DisplayBuffer[1] );
				u8g_DrawStr( &OLED_Instance, 0, 20, DisplayBuffer[2] );
				u8g_DrawStr( &OLED_Instance, 0, 30, DisplayBuffer[3] );
			}
			while (u8g_NextPage(&OLED_Instance));			
		}

		if (TC4_IsOverflow)
		{
			TC4_IsOverflow = 0;
			LED2_Toggle();
			
			Duty += DutyDistance;
			if (Duty >= 100 || Duty <= 0)
				DutyDistance = -DutyDistance;

			if (Duty >= 100)
				TCC2_PWM16bitDutySet( 1, TCC2_PWM16bitPeriodGet() + 1 );
			else
				TCC2_PWM16bitDutySet( 1, ((uint32_t)Duty * TCC2_PWM16bitPeriodGet()) / 100 );			
		}

		if( USART5_IsReceived )
		{
			USART5_IsReceived = 0;

			printf( "\r\nReceived Data : %1c\r\n", USART5_ReceiveData[0] );
			SERCOM5_USART_Read( USART5_ReceiveData, 1 );
			
			// TODO 17.06
			sprintf( (char *) DisplayBuffer[3], "Received Data : %1c", USART5_ReceiveData[0] );			
		}

		if( ADC_IsCompleted )
		{
			ADC_IsCompleted = 0;

			printf( "VR1 Value : %4d\r\n", ADC_Result[0]);
			printf( "Temperature Value : %4d\r\n", ADC_Result[1]);
			
			// TODO 17.07
			sprintf( (char *) DisplayBuffer[1], "VR1 Value : %4d", ADC_Result[0] );
			sprintf( (char *) DisplayBuffer[2], "Temp. Value: %4d", ADC_Result[1] );			
		}		
	}

	/* Execution should not come here during normal operation */

	return ( EXIT_FAILURE );
}


/*******************************************************************************
 End of File
*/

