/*******************************************************************************
  Main Source File

  Company:
    Microchip Technology Inc.

  File Name:
    main.c

  Summary:
    This file contains the "main" function for a project.

  Description:
    This file contains the "main" function for a project.  The
    "main" function calls the "SYS_Initialize" function to initialize the state
    machines of all modules in the system
 *******************************************************************************/

// *****************************************************************************
// *****************************************************************************
// Section: Included Files
// *****************************************************************************
// *****************************************************************************

#include <stddef.h>                     // Defines NULL
#include <stdbool.h>                    // Defines true
#include <stdlib.h>                     // Defines EXIT_FAILURE
#include "definitions.h"                // SYS function prototypes
#include <stdio.h>
#include <string.h>

// *****************************************************************************
// *****************************************************************************
// Section: Main Entry Point
// *****************************************************************************
// *****************************************************************************
uint32_t i = 0;

volatile uint8_t TC3_IsOverflow = 0;
volatile uint8_t TC4_IsOverflow = 0;
uint8_t USARTRTxBuffer[100];
uint8_t USART5_ReceiveData[1];
volatile uint8_t USART5_IsReceived = 0;
volatile uint8_t USART5_IsTransmitted = 1;
uint16_t ADC_Result[2];
volatile uint8_t ADC_IsCompleted = 0;
volatile int8_t ADC_ChannelIdx = 0;

void ADC_Complete( ADC_STATUS status, uintptr_t context )
{		
	if( status & ADC_INTFLAG_RESRDY_Msk )
	{		
		ADC_Result[ADC_ChannelIdx] = ADC_ConversionResultGet();	
		ADC_ChannelIdx = ( ADC_ChannelIdx + 1 ) % 2;
		if( ADC_ChannelIdx==0 )
		{
			ADC_IsCompleted=1;
		}
	}
}

void USART5_Transmit( uintptr_t context )
{
	USART5_IsTransmitted = 1;
}

void USART5_Receive( uintptr_t context )
{
	USART5_IsReceived = 1;
}

void TC3_Overflow(TC_COMPARE_STATUS status, uintptr_t context) /* CallBack */
{
	if( status & TC_INTFLAG_OVF_Msk )
	{	
		TC3_IsOverflow = 1;
	}
}

void TC4_Overflow(TC_COMPARE_STATUS status, uintptr_t context) /* CallBack */
{
	if( status & TC_INTFLAG_OVF_Msk )
	{
		TC4_IsOverflow = 1;
	}
}

int main ( void )
{
	/* Initialize all modules */
	SYS_Initialize ( NULL );

	TC3_CompareCallbackRegister( TC3_Overflow, (uintptr_t )NULL );
	TC3_CompareStart();

	TC4_CompareCallbackRegister( TC4_Overflow, (uintptr_t )NULL );
	TC4_CompareStart();	

	SERCOM5_USART_ReadCallbackRegister( USART5_Receive, (uintptr_t )NULL );
	SERCOM5_USART_WriteCallbackRegister( USART5_Transmit, (uintptr_t )NULL );
	SERCOM5_USART_Read( USART5_ReceiveData, 1 );

	ADC_CallbackRegister( ADC_Complete, (uintptr_t )NULL );
	ADC_Enable();

	TCC2_PWMStart();

	while ( true )
	{
		/* Maintain state machines of all polled MPLAB Harmony modules. */
		SYS_Tasks ( );

		if (TC3_IsOverflow)
		{
			TC3_IsOverflow = 0;
			LED1_Toggle();
			printf( "Hello World!!\r\n" );

			ADC_ConversionStart();
		}

		if (TC4_IsOverflow)
		{
			TC4_IsOverflow = 0;
			LED2_Toggle();
		}

		if( USART5_IsReceived )
		{
			USART5_IsReceived = 0;

			printf( "\r\nReceived Data : %1c\r\n", USART5_ReceiveData[0] );
			SERCOM5_USART_Read( USART5_ReceiveData, 1 );
		}

		if( ADC_IsCompleted )
		{
			ADC_IsCompleted = 0;

			printf( "VR1 Value : %4d\r\n", ADC_Result[0]);
			printf( "Temperature Value : %4d\r\n", ADC_Result[1]);
			
		    // TODO 15.01
			
			
		}		
	}

	/* Execution should not come here during normal operation */

	return ( EXIT_FAILURE );
}


/*******************************************************************************
 End of File
*/

