/*******************************************************************************
  Main Source File

  Company:
    Microchip Technology Inc.

  File Name:
    main.c

  Summary:
    This file contains the "main" function for a project.

  Description:
    This file contains the "main" function for a project.  The
    "main" function calls the "SYS_Initialize" function to initialize the state
    machines of all modules in the system
 *******************************************************************************/

// *****************************************************************************
// *****************************************************************************
// Section: Included Files
// *****************************************************************************
// *****************************************************************************

#include <stddef.h>                     // Defines NULL
#include <stdbool.h>                    // Defines true
#include <stdlib.h>                     // Defines EXIT_FAILURE
#include "definitions.h"                // SYS function prototypes
#include <stdio.h>
#include <string.h>

// *****************************************************************************
// *****************************************************************************
// Section: Main Entry Point
// *****************************************************************************
// *****************************************************************************
uint32_t i = 0;

// TODO 11.01
volatile uint8_t TC3_IsOverflow = 0;
volatile uint8_t TC4_IsOverflow = 0;

uint8_t USARTRTxBuffer[100];
uint8_t USART5_ReceiveData[1];
volatile uint8_t USART5_IsReceived = 0;
volatile uint8_t USART5_IsTransmitted = 1;

void USART5_Transmit( uintptr_t context )
{
	USART5_IsTransmitted = 1;
}

void USART5_Receive( uintptr_t context )
{
	USART5_IsReceived = 1;
}

void TC3_Overflow(TC_COMPARE_STATUS status, uintptr_t context) /* CallBack */
{
	if( status & TC_INTFLAG_OVF_Msk )
	{	
		// TODO 11.02
		TC3_IsOverflow = 1;
	}
}

void TC4_Overflow(TC_COMPARE_STATUS status, uintptr_t context) /* CallBack */
{
	if( status & TC_INTFLAG_OVF_Msk )
	{
		// TODO 11.03
		TC4_IsOverflow = 1;
	}
}

int main ( void )
{
	/* Initialize all modules */
	SYS_Initialize ( NULL );

	TC3_CompareCallbackRegister( TC3_Overflow, (uintptr_t )NULL );
	TC3_CompareStart();

	TC4_CompareCallbackRegister( TC4_Overflow, (uintptr_t )NULL );
	TC4_CompareStart();	

	SERCOM5_USART_ReadCallbackRegister( USART5_Receive, (uintptr_t )NULL );
	SERCOM5_USART_WriteCallbackRegister( USART5_Transmit, (uintptr_t )NULL );
	SERCOM5_USART_Read( USART5_ReceiveData, 1 );
	
	while ( true )
	{
		/* Maintain state machines of all polled MPLAB Harmony modules. */
		SYS_Tasks ( );

		// TODO 11.04
		if (TC3_IsOverflow)
		{
			TC3_IsOverflow = 0;
			LED1_Toggle();
			printf( "Hello World!!\r\n" );
		}

		if (TC4_IsOverflow)
		{
			TC4_IsOverflow = 0;
			LED2_Toggle();
		}

		if( USART5_IsReceived )
		{
			USART5_IsReceived = 0;

			// TODO 11.05
			printf( "\r\nReceived Data : %1c\r\n", USART5_ReceiveData[0] );

			SERCOM5_USART_Read( USART5_ReceiveData, 1 );
		}		
	}

	/* Execution should not come here during normal operation */

	return ( EXIT_FAILURE );
}


/*******************************************************************************
 End of File
*/

